import React, { ReactElement, SVGProps } from 'react';
import { ChartOffset, D3Scale } from '../util/types';
import { Props as XAxisProps } from './XAxis';
import { Props as YAxisProps } from './YAxis';
export declare type GridLineTypeFunctionProps = Omit<LineItemProps, 'key'> & {
    key: LineItemProps['key'] | undefined;
    offset: ChartOffset;
};
declare type GridLineType = SVGProps<SVGLineElement> | ReactElement<SVGElement> | ((props: GridLineTypeFunctionProps) => ReactElement<SVGElement>) | boolean;
export declare type HorizontalCoordinatesGenerator = (props: {
    yAxis: any;
    width: number;
    height: number;
    offset: ChartOffset;
}, syncWithTicks: boolean) => number[];
export declare type VerticalCoordinatesGenerator = (props: {
    xAxis: any;
    width: number;
    height: number;
    offset: ChartOffset;
}, syncWithTicks: boolean) => number[];
interface InternalCartesianGridProps {
    width?: number;
    height?: number;
    horizontalCoordinatesGenerator?: HorizontalCoordinatesGenerator;
    verticalCoordinatesGenerator?: VerticalCoordinatesGenerator;
    xAxis?: null | (Omit<XAxisProps, 'scale'> & {
        scale: D3Scale<string | number>;
    });
    yAxis?: null | (Omit<YAxisProps, 'scale'> & {
        scale: D3Scale<string | number>;
    });
}
interface CartesianGridProps extends InternalCartesianGridProps {
    x?: number;
    y?: number;
    horizontal?: GridLineType;
    vertical?: GridLineType;
    horizontalPoints?: number[];
    verticalPoints?: number[];
    verticalFill?: string[];
    horizontalFill?: string[];
    syncWithTicks?: boolean;
    horizontalValues?: number[] | string[];
    verticalValues?: number[] | string[];
}
declare type AcceptedSvgProps = Omit<SVGProps<SVGElement>, 'offset'>;
export declare type Props = AcceptedSvgProps & CartesianGridProps;
declare type LineItemProps = Props & {
    offset: ChartOffset;
    x1: number;
    y1: number;
    x2: number;
    y2: number;
    key: string;
    index: number;
};
export declare function CartesianGrid(props: Props): React.JSX.Element;
export declare namespace CartesianGrid {
    var displayName: string;
}
export {};
